import { Injectable } from '@nestjs/common';
import { CreateMovieDto } from './dto/create-movie.dto';
import { UpdateMovieDto } from './dto/update-movie.dto';
import { PrismaService } from '../../prisma/prisma.service';
import { plainToClass } from 'class-transformer';
import { MovieEntity } from './entities/movie.entity';
import { Movie } from '@prisma/client';

@Injectable()
export class MoviesService {
  constructor(private prisma: PrismaService) {}

  async create(createMovieDto: CreateMovieDto): Promise<Movie> {
    const movie = plainToClass(MovieEntity, createMovieDto);
    return await this.prisma.movie.create({
      data: movie,
    });
  }

  async findAll(mostRecentCnt?: number): Promise<Movie[]> {
    if (mostRecentCnt) {
      return await this.prisma.movie.findMany();
    }

    return await this.prisma.movie.findMany({
      orderBy: {
        publishedAt: 'desc',
      },
      take: mostRecentCnt,
      include: {
        director: true,
      },
    });
  }

  async findOne(id: string): Promise<Movie> {
    return await this.prisma.movie.findUnique({
      where: {
        id,
      },
      include: {
        director: true,
      },
    });
  }

  async addCategory(movieId: string, categoryName: string): Promise<Movie> {
    await this.prisma.category.update({
      data: {
        movies: {
          connect: {
            id: movieId,
          },
        },
      },
      where: {
        name: categoryName,
      },
    });

    return await this.prisma.movie.update({
      data: {
        categories: {
          connect: {
            name: categoryName,
          },
        },
      },
      where: {
        id: movieId,
      },
      include: {
        categories: true,
      },
    });
  }

  async update(id: string, updateMovieDto: UpdateMovieDto): Promise<Movie> {
    const updatedMovie = {
      ...(await this.findOne(id)),
      ...plainToClass(MovieEntity, updateMovieDto),
    };
    return await this.prisma.movie.update({
      where: {
        id,
      },
      data: {
        ...updatedMovie,
      },
    });
  }

  async remove(id: string): Promise<Movie> {
    return await this.prisma.movie.update({
      where: {
        id,
      },
      data: {
        deletedAt: new Date(),
      },
    });
  }
}
